@@ -14,6 +14,7 @@ from .core.settings import SETTINGS as settings
 from .datasets import Dataset
 from .datasets import get_dataset as dataset  # so the user can do: cml.dataset(...)
 from .datasets import load_dataset
+from .distributed.dask import start_dask
 from .plotting import interactive_map, new_plot, new_table
 from .plotting import options as plotting_options
 from .plotting import plot_graph, plot_map
@@ -43,6 +44,7 @@ __all__ = [
     "settings",
     "source",
     "Source",
+    "start_dask",
     "__version__",
 ]
 
