@@ -19,6 +19,7 @@ decide to read it or create it.
 
 import ctypes
 import datetime
+import functools
 import hashlib
 import json
 import logging
@@ -28,8 +29,8 @@ import shutil
 import sqlite3
 import threading
 import time
-from functools import wraps
 
+import pandas as pd
 from filelock import FileLock
 
 from climetlab.core.settings import SETTINGS
@@ -90,11 +91,15 @@ def disk_usage(path):
 def default_serialiser(o):
     if isinstance(o, (datetime.date, datetime.datetime)):
         return o.isoformat()
+    if isinstance(o, (pd.Timestamp)):
+        return o.isoformat()
+    if isinstance(o, (pd.DatetimeIndex)):
+        return [_.isoformat() for _ in o]
     return json.JSONEncoder.default(o)
 
 
 def in_executor(func):
-    @wraps(func)
+    @functools.wraps(func)
     def wrapped(*args, **kwargs):
         global CACHE
         s = CACHE.enqueue(func, *args, **kwargs)
@@ -104,7 +109,7 @@ def in_executor(func):
 
 
 def in_executor_forget(func):
-    @wraps(func)
+    @functools.wraps(func)
     def wrapped(*args, **kwargs):
         global CACHE
         CACHE.enqueue(func, *args, **kwargs)
@@ -368,7 +373,8 @@ class Cache(threading.Thread):
                 shutil.rmtree(path)
             else:
                 os.unlink(path)
-        except Exception:
+        except Exception as e:
+            print(e)
             LOG.exception("Deleting %s", path)
 
     def _entry_to_dict(self, entry):
