@@ -7,12 +7,12 @@
 # nor does it submit to any jurisdiction.
 #
 
+import functools
 import getpass
 import logging
 import os
 import tempfile
 from contextlib import contextmanager
-from functools import wraps
 from typing import Callable
 
 import yaml
@@ -75,6 +75,11 @@ SETTINGS_AND_HELP = {
         """List of directories where to search for layers definitions.
         See :ref:`layers` for more information.""",
     ),
+    "dask-directories": _(
+        [os.path.join(DOT_CLIMETLAB, "dask")],
+        """List of directories where to search for dask cluster definitions.
+        See :ref:`dask` for more information.""",
+    ),
     "datasets-directories": _(
         [os.path.join(DOT_CLIMETLAB, "datasets")],
         """List of directories where to search for datasets definitions.
@@ -151,7 +156,7 @@ def new_settings(s):
 
 
 def forward(func):
-    @wraps(func)
+    @functools.wraps(func)
     def wrapped(self, *args, **kwargs):
         if self._stack:
             return func(self._stack[-1], *args, **kwargs)
