@@ -58,9 +58,6 @@ class Reader(Base, os.PathLike, metaclass=ReaderMeta):
         # Used by multi-source
         return False
 
-    def sel(self, *args, **kwargs):
-        raise NotImplementedError()
-
     def cache_file(self, *args, **kwargs):
         return self.source.cache_file(*args, **kwargs)
 
@@ -83,6 +80,10 @@ class Reader(Base, os.PathLike, metaclass=ReaderMeta):
     def __fspath__(self):
         return self.path
 
+    def index_content(self):
+        LOG.warning(f"index-content(): Ignoring {self.path}")
+        return []
+
 
 _READERS = {}
 
