@@ -8,14 +8,15 @@
 #
 
 import datetime
-import json
 import logging
 import os
+import threading
+import time
+from itertools import islice
 
 import eccodes
 
 from climetlab.core import Base
-from climetlab.core.caching import auxiliary_cache_file
 from climetlab.profiling import call_counter
 from climetlab.utils.bbox import BoundingBox
 
@@ -27,7 +28,7 @@ def missing_is_none(x):
 
 
 # This does not belong here, should be in the C library
-def _get_message_offsets(path):
+def get_messages_positions(path):
 
     fd = os.open(path, os.O_RDONLY)
     try:
@@ -98,14 +99,48 @@ class CodesHandle:
         self.offset = offset
 
     def __del__(self):
-        eccodes_codes_release(self.handle)
+        try:
+            eccodes_codes_release(self.handle)
+        except TypeError:
+            # This happens when eccodes is unloaded before
+            # this object is deleted
+            pass
 
     def get(self, name):
         try:
             if name == "values":
                 return eccodes.codes_get_values(self.handle)
             size = eccodes.codes_get_size(self.handle, name)
-            LOG.debug(f"{name}:{size}")
+            # LOG.debug(f"{name}:{size}")
+
+            if name == "md5GridSection":
+
+                # Special case because:
+                #
+                # 1) eccodes is returning size > 1 for 'md5GridSection'
+                # (size = 16 : it is the number of bytes of the value)
+                # This will be fixed in eccodes.
+                #
+                # 2) sometimes (see below), the value for "shapeOfTheEarth" is inconsistent.
+                # This impacts the (computed on-the-fly) value of "md5GridSection".
+                # ----------------
+                # Example of data with inconsistent values:
+                # S2S data, origin='ecmf', param='tp', step='24', number='0', date=['20201203','20200702']
+                # the 'md5GridSection' are different
+                # This is because one has "shapeOfTheEarth" set to 0, the other to 6.
+                # This is only impacting the metadata.
+                # Since this has no impact on the data itself,
+                # this is unlikely to be fixed. Therefore this hacky patch.
+                #
+                # Obviously, the patch causes an inconsistency between the value of md5GridSection
+                # read by this code, and the value read by another code without this patch.
+
+                save = eccodes.codes_get_long(self.handle, "shapeOfTheEarth")
+                eccodes.codes_set_long(self.handle, "shapeOfTheEarth", 255)
+                result = eccodes.codes_get_string(self.handle, "md5GridSection")
+                eccodes.codes_set_long(self.handle, "shapeOfTheEarth", save)
+                return result
+
             if size and size > 1:
                 return eccodes.codes_get_array(self.handle, name)
             return eccodes.codes_get(self.handle, name)
@@ -130,70 +165,115 @@ class CodesHandle:
         except eccodes.KeyValueNotFoundError:
             return None
 
+    def get_data(self):
+        return eccodes.codes_grib_get_data(self.handle)
+
+    def as_mars(self, param="shortName"):
+        r = {}
+        it = eccodes.codes_keys_iterator_new(self.handle, "mars")
+
+        try:
+            while eccodes.codes_keys_iterator_next(it):
+                key = eccodes.codes_keys_iterator_get_name(it)
+                r[key] = self.get(param if key == "param" else key)
+        finally:
+            eccodes.codes_keys_iterator_delete(it)
+
+        return r
+
+    def clone(self):
+        return CodesHandle(eccodes.codes_clone(self.handle), None, None)
+
+    def set_values(self, values):
+        assert self.path is None, "Only cloned handles can have values changed"
+        eccodes.codes_set_values(self.handle, values.flatten())
+        eccodes.codes_set_long(self.handle, "generatingProcessIdentifier", 254)
+
+    def save(self, path):
+        with open(path, "wb") as f:
+            eccodes.codes_write(self.handle, f)
+            self.path = path
+            self.offset = 0
+
+    def read_bytes(self, offset, length):
+        with open(self.path, "rb") as f:
+            f.seek(offset)
+            return f.read(length)
+
+
+class ReaderLRUCache(dict):
+    def __init__(self, size):
+        self.readers = dict()
+        self.lock = threading.Lock()
+        self.size = size
+
+    def __getitem__(self, path):
+        with self.lock:
+            try:
+                return super().__getitem__(path)
+            except KeyError:
+                pass
+
+            c = self[path] = CodesReader(path)
+            while len(self) >= self.size:
+                oldest = min((v.last, v.path) for v in self.values())
+                del self[oldest[1]]
+
+            return c
+
+
+cache = ReaderLRUCache(512)  # TODO: Add to config
+
 
 class CodesReader:
     def __init__(self, path):
         self.path = path
+        self.lock = threading.Lock()
+        # print("OPEN", self.path)
         self.file = open(self.path, "rb")
+        self.last = time.time()
 
     def __del__(self):
         try:
+            # print("CLOSE", self.path)
             self.file.close()
         except Exception:
             pass
 
+    @classmethod
+    def from_cache(cls, path):
+        return cache[path]
+
     def at_offset(self, offset):
-        self.file.seek(offset, 0)
-        return next(self)
-
-    def __iter__(self):
-        return self
-
-    def __next__(self):
-        handle = self._next_handle()
-        if handle is None:
-            raise StopIteration()
-        return handle
-
-    def _next_handle(self):
-        offset = self.file.tell()
-        handle = eccodes_codes_new_from_file(self.file, eccodes.CODES_PRODUCT_GRIB)
-        if not handle:
-            return None
-        return CodesHandle(handle, self.path, offset)
-
-    @property
-    def offset(self):
-        return self.file.tell()
-
-    def read(self, offset, length):
-        self.file.seek(offset, 0)
-        return self.file.read(length)
+        with self.lock:
+            self.last = time.time()
+            self.file.seek(offset, 0)
+            handle = eccodes_codes_new_from_file(
+                self.file,
+                eccodes.CODES_PRODUCT_GRIB,
+            )
+            assert handle is not None
+            return CodesHandle(handle, self.path, offset)
 
 
 class GribField(Base):
-    def __init__(self, reader, offset, length):
-        self._reader = reader
+    def __init__(self, path, offset, length):
+        self.path = path
         self._offset = offset
         self._length = length
         self._handle = None
 
-    def __enter__(self):
-        return self
+    # def __enter__(self):
+    #     return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        pass
-
-    @property
-    def path(self):
-        return self.handle.path
+    # def __exit__(self, exc_type, exc_val, exc_tb):
+    #     pass
 
     @property
     def handle(self):
         if self._handle is None:
             assert self._offset is not None
-            assert self._reader is not None
-            self._handle = self._reader.at_offset(self._offset)
+            self._handle = CodesReader.from_cache(self.path).at_offset(self._offset)
         return self._handle
 
     @property
@@ -208,10 +288,11 @@ class GribField(Base):
 
     @property
     def shape(self):
-        return (
-            missing_is_none(self.handle.get("Nj")),
-            missing_is_none(self.handle.get("Ni")),
-        )
+        Nj = missing_is_none(self.handle.get("Nj"))
+        Ni = missing_is_none(self.handle.get("Ni"))
+        if Ni is None or Nj is None:
+            return self.handle.get("numberOfDataPoints")
+        return (Nj, Ni)
 
     def plot_map(self, backend):
         backend.bounding_box(
@@ -223,12 +304,7 @@ class GribField(Base):
         backend.plot_grib(self.path, self.handle.get("offset"))
 
     @call_counter
-    def to_numpy(self, normalise=False):
-        shape = self.shape
-        if shape[0] is None or shape[1] is None:
-            return self.values
-        if normalise:
-            return self.values.reshape(self.shape)
+    def to_numpy(self):
         return self.values.reshape(self.shape)
 
     def __repr__(self):
@@ -300,11 +376,16 @@ class GribField(Base):
             name = "paramId"
         return self.handle.get(name)
 
+    def metadata(self, name):
+        if name == "param":
+            name = "shortName"
+        if name == "_param_id":
+            name = "paramId"
+        return self[name]
+
     def __getitem__(self, name):
         """For cfgrib"""
 
-        # print(name)
-
         proc = self.handle.get
         if ":" in name:
             try:
@@ -320,69 +401,41 @@ class GribField(Base):
 
         return proc(name)
 
+    @property
+    def data(self):
+        return self.handle.get_data()
+
+    def as_mars(self, param="shortName"):
+        return self.handle.as_mars(param)
+
     def write(self, f):
-        f.write(self._reader.read(self._offset, self._length))
+        f.write(self.handle.read_bytes(self._offset, self._length))
 
+    def plot_numpy(self, backend, array):
 
-class GribIndex:
+        if self.handle.get("gridType") == "regular_ll":
+            metadata = self.field_metadata()
 
-    VERSION = 1
+            backend.bounding_box(
+                north=metadata["north"],
+                south=metadata["south"],
+                west=metadata["west"],
+                east=metadata["east"],
+            )
 
-    def __init__(self, path):
-        assert isinstance(path, str), path
-        self.path = path
-        self.offsets = None
-        self.lengths = None
-        self.cache = auxiliary_cache_file(
-            "grib-index",
-            path,
-            content="null",
-            extension=".json",
-        )
+            backend.plot_numpy(
+                array.reshape(metadata.get("shape", self.shape)),
+                metadata=metadata,
+            )
+            return
 
-        if not self._load_cache():
-            self._build_index()
+        # Non-regular field
+        tmp = backend.temporary_file(".grib")
+        clone = self.handle.clone()
+        clone.set_values(array)
+        clone.save(tmp)
+        GribField(tmp, 0, self._length).plot_map(backend)
 
-    def _build_index(self):
-
-        offsets = []
-        lengths = []
-
-        for offset, length in _get_message_offsets(self.path):
-            offsets.append(offset)
-            lengths.append(length)
-
-        self.offsets = offsets
-        self.lengths = lengths
-
-        self._save_cache()
-
-    def _save_cache(self):
-        try:
-            with open(self.cache, "w") as f:
-                json.dump(
-                    dict(
-                        version=self.VERSION,
-                        offsets=self.offsets,
-                        lengths=self.lengths,
-                    ),
-                    f,
-                )
-        except Exception:
-            LOG.exception("Write to cache failed %s", self.cache)
-
-    def _load_cache(self):
-        try:
-            with open(self.cache) as f:
-                c = json.load(f)
-                if not isinstance(c, dict):
-                    return False
-
-                assert c["version"] == self.VERSION
-                self.offsets = c["offsets"]
-                self.lengths = c["lengths"]
-                return True
-        except Exception:
-            LOG.exception("Load from cache failed %s", self.cache)
-
-        return False
+    def iterate_grid_points(self):
+        latlon = self.handle.get("latitudeLongitudeValues")
+        yield from zip(islice(latlon, 0, None, 3), islice(latlon, 1, None, 3))
