@@ -9,18 +9,18 @@
 
 import logging
 
-from .. import Reader
-from .fieldset import FieldSet
+from climetlab.readers import Reader
+from climetlab.readers.grib.index import FieldSetInOneFile, MultiFieldSet
 
 LOG = logging.getLogger(__name__)
 
 
-class GRIBReader(FieldSet, Reader):
+class GRIBReader(FieldSetInOneFile, Reader):
     appendable = True  # GRIB messages can be added to the same file
 
     def __init__(self, source, path):
         Reader.__init__(self, source, path)
-        FieldSet.__init__(self, paths=[path])
+        FieldSetInOneFile.__init__(self, path)
 
     def __repr__(self):
         return "GRIBReader(%s)" % (self.path,)
@@ -31,4 +31,10 @@ class GRIBReader(FieldSet, Reader):
         assert all(isinstance(s, GRIBReader) for s in readers), readers
         assert len(readers) > 1
 
-        return FieldSet(paths=[r.path for r in readers])
+        return MultiFieldSet(readers)
+
+    def index_content(self):
+        assert False, "not used"
+        from climetlab.readers.grib.parsing import _index_grib_file
+
+        yield from _index_grib_file(self.path)
