@@ -187,14 +187,20 @@ class CheckCmd:
                             if module:
                                 modules.add(module)
 
+        modules.add("dask")
+        modules.add("distributed")
+        modules.add("asyncssh")
         return modules
 
     @parse_args(
-        modules=dict(
-            metavar="PACKAGE",
-            type=str,
-            nargs="*",
-            help="optional list of Python packages",
+        modules=(
+            "modules",
+            dict(
+                metavar="PACKAGE",
+                type=str,
+                nargs="*",
+                help="optional list of Python packages",
+            ),
         ),
         json=dict(action="store_true", help="produce a JSON output"),
         all=dict(action="store_true"),
