@@ -84,8 +84,11 @@ class FileSource(Source, os.PathLike, metaclass=FileSourceMeta):
     def __getitem__(self, n):
         return self._reader[n]
 
-    def sel(self, **kwargs):
-        return self._reader.sel(**kwargs)
+    def sel(self, *args, **kwargs):
+        return self._reader.sel(*args, **kwargs)
+
+    def order_by(self, *args, **kwargs):
+        return self._reader.order_by(*args, **kwargs)
 
     def plot_graph(self, *args, **kwargs):
         return self._reader.plot_graph(*args, **kwargs)
@@ -115,6 +118,9 @@ class FileSource(Source, os.PathLike, metaclass=FileSourceMeta):
     def write(self, f):
         return self._reader.write(f)
 
+    def scaled(self, *args, **kwargs):
+        return self._reader.scaled(*args, **kwargs)
+
     def _attributes(self, names):
         return self._reader._attributes(names)
 
