@@ -17,8 +17,12 @@ class NumpyArrayWrapper(Wrapper):
 
     def plot_map(self, backend):
 
-        metadata = get_wrapper(backend.option("metadata"))
-        metadata = metadata.field_metadata()
+        wrapper = get_wrapper(backend.option("metadata"))
+
+        if hasattr(wrapper, "plot_numpy"):
+            return wrapper.plot_numpy(backend, self.data)
+
+        metadata = wrapper.field_metadata()
 
         backend.bounding_box(
             north=metadata["north"],
